//
//  ViewController.m
//  WebKit
//
//  Created by Clemens Wagner on 08.08.15.
//  Copyright © 2015 Clemens Wagner. All rights reserved.
//

#import "WKWebViewController.h"
#import <WebKit/WebKit.h>
#import "NSString+Extensions.h"

static NSString * const kHeader = @"<style>form { display: none; height: 0; top: 0; } form input { position: absolute; height: 0; top: 0; } ul.ui-autocomplete { width: 100%; background-color: white; } ul.ui-autocomplete li.ui-menu-item { font-weight: 400; padding: 8px; border-bottom: 1px solid black; }</style>\n";

@interface WKWebViewController ()<UITextFieldDelegate, WKNavigationDelegate>

@property (weak, nonatomic) WKWebView *webView;
@property (weak, nonatomic) IBOutlet UITextField *queryField;
@property (nonatomic) BOOL loadDirect;

@end

@implementation WKWebViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    UIView *theView = self.view;
    CGRect theFrame = theView.bounds;
    WKWebView *theWebView = [[WKWebView alloc] initWithFrame:theFrame];
    NSURL *theURL = [NSURL URLWithString:@"http://localhost:8080/"];
    NSURLRequest *theRequest = [NSURLRequest requestWithURL:theURL];
    
    theWebView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    theWebView.navigationDelegate = self;
    [theView addSubview:theWebView];
    self.webView = theWebView;
    [self loadRequest:theRequest];
}

- (void)loadRequest:(NSURLRequest *)inRequest {
    NSURLSession *theSession = [NSURLSession sharedSession];
    NSURLSessionTask *theTask = [theSession dataTaskWithRequest:inRequest completionHandler:^(NSData *inData, NSURLResponse * inResponse, NSError *inError) {
        if(inData == nil) {
            NSLog(@"loadRequest: failed (%@)", inError);
        }
        else {
            NSString *theTextEncoding = inResponse.textEncodingName;
            NSStringEncoding theEncoding = theTextEncoding == nil ? NSUTF8StringEncoding : [theTextEncoding stringEncoding];
            NSString *theContent = [[NSString alloc] initWithData:inData encoding:theEncoding];
            NSRange theRange = [theContent rangeOfString:@"</head>"];
            
            theContent = [theContent stringByReplacingCharactersInRange:NSMakeRange(theRange.location, 0) withString:kHeader];
            self.loadDirect = YES;
            [self.webView loadHTMLString:theContent baseURL:inRequest.URL];
        }
    }];
    
    [theTask resume];
}

#pragma mark UITextFieldDelegate

- (void)textFieldDidBeginEditing:(UITextField *)inTextField {
    [self.webView.scrollView setContentOffset:CGPointZero animated:YES];
    [self.webView evaluateJavaScript:@"$('#search input[name=q]').keydown().keyup();" completionHandler:NULL];
}

- (BOOL)textField:(UITextField *)inTextField shouldChangeCharactersInRange:(NSRange)inRange replacementString:(NSString *)inText {
    if([inText isEqualToString:@"\n"]) {
        [self.webView evaluateJavaScript:@"document.forms.search.submit();" completionHandler:^(id inResult, NSError *inError) {
            NSLog(@"error = %@", inError);
            [inTextField endEditing:YES];
        }];
    }
    else {
        NSString *theText = [inTextField.text stringByReplacingCharactersInRange:inRange withString:inText];
        NSString *theScript = [NSString stringWithFormat:@"$('#search input[name=q]').val(%@).keydown().keyup();",
                               theText.javaScriptString];
        
        [self.webView evaluateJavaScript:theScript completionHandler:^(id inResult, NSError *inError) {
            NSLog(@"error = %@", inError);
        }];
    }
    return YES;
    
}

#pragma mark WKNavigationDelegate

- (void)webView:(WKWebView *)inWebView decidePolicyForNavigationAction:(WKNavigationAction *)inAction decisionHandler:(void (^)(WKNavigationActionPolicy))inHandler {
    if(self.loadDirect) {
        self.loadDirect = NO;
        inHandler(WKNavigationActionPolicyAllow);
    }
    else {
        inHandler(WKNavigationActionPolicyCancel);
        [self loadRequest:inAction.request];
    }
}

-(void)webView:(WKWebView *)inWebView didFinishNavigation:(WKNavigation *)inNavigation {
    [inWebView evaluateJavaScript:@"$('#search input[name=q]').autocomplete({ open: function(event, ui) { $('.ui-autocomplete').off('menufocus hover mouseover mouseenter');}});" completionHandler:^(id inResult, NSError *inError) {
    }];
}

@end
